#! /usr/bin/python
""" input: i) GG file (e.g. from PPI);  ii) Gene2Fun2Source ; iii) output directory ; iv) [optional] minimum # of Genes in a function/pathway to be considered ; v) [optional] maximum # of Genes in a function/pathway to be considered ;  vi) [optional] using full shortest path using all genes from the GGfile (F) or only computing shortest path using the fun genes in GGfile (f)

### vi) [optional] minimum # of Genes in a function/pathway overlapping with the GG file to be considered ;

output: in the output directory (iii), create subdirectories (A-Z and 0-9), and store each function/pathway's Shortest PATH file (G1\tG2\tshortestpathdist\tshortestpath), with headers:
#Name
#Source
#TG_count (total genes in function count)
!!! ONLY RETURN RESULTS IF THERE IS PATH BETWEEN A AND B
!!! ONLY RETURN either A2B or B2A !!!
"""
from optparse import OptionParser
import sys
import os
sys.path.append("/net/fantasia/home/alextsoi/code/Network")
from ggscoreGraph import *

def main():
	usage="usage: %prog -i GGfile -g gene2fun2source -o outDir [-m minFun -M maxFun -a minFunNet -F/f]"
	parser=OptionParser(usage)
	parser.add_option("-i", "--GGfile",dest="GGfile",help="file name of input GG file")
	parser.add_option("-g", "--annofile",dest="annofile",help="file name of annotation file (gene2anno2source)")
	parser.add_option("-o", "--outDir",dest="outDir",help="output directory")
	
	### default value for m, M, and a are 5, 1000, and 3
	parser.add_option("-m","--minFun",dest="minFun",type="int",default=5)
	parser.add_option("-M","--maxFun",dest="maxFun",type="int",default=1000)
	#parser.add_option("-a","--minFunNet",dest="minFunNet",type="int",default=3)

	### using all genes in GGfile or using all the fun genes in GGfile to comute SP
	parser.add_option("-F","--fullgenes",action="store_true",dest="fullgenes",default=True,help="using all genes in GGfile to compute SP (default)")
	parser.add_option("-f","--fungenes",action="store_false",dest="fullgenes",help="using the fun genes in GGfile to compute SP")


	(options,args)=parser.parse_args()
	if options.GGfile==None or options.annofile==None or options.outDir==None:
		parser.error("SPfile or annofile or outDir not specified")

	############################################################
	os.chdir(options.outDir)
	graph=unweightedGGGraph(options.GGfile)
	minFun=options.minFun
	maxFun=options.maxFun
	#minFunNet=options.minFunNet
	fullgenes=options.fullgenes
	print "Minimum Function size set to be: "+str(minFun)
	print "Maximum Function size set to be: "+str(maxFun)
	#print "Minimum overlap between Function and Interaction network set to be: "+str(minFunNet)
	print "Using all genes in GGfile to compute SP: "+str(fullgenes)
	###################################################################
	#### parse the annofile: 
	annoDict={}
	count=0
	for tempstr in open(options.annofile):
		count+=1
		if (count%500000) ==0:
			print "Processed "+str(count)+" gene2function reocrds..."
		tempstr=tempstr.rstrip().split()
		if tempstr[1] not in annoDict:
			### storing each function source; total gene count; and the genes in fun
			annoDict[tempstr[1]]=[tempstr[2],1,[tempstr[0]]]
		else:
			annoDict[tempstr[1]][1]+=1
			annoDict[tempstr[1]][2].append(tempstr[0])
	###############################################################
	### generate file handlers for each function
	funhandlers={}
	for fun in annoDict:
		pre=fun[0]
		if pre not in funhandlers:
			funhandlers[pre]={}
			os.system("mkdir "+pre)
		### create the SP file for the function
		funhandlers[pre][fun]=open(pre+"/"+fun,'w')
		funhandlers[pre][fun].write("###"+fun+"\n###"+annoDict[fun][0]+"\n###TG_count="+str(annoDict[fun][1])+"\n")
	#####################################################
	### generate SP records for each function
	graphgenes=graph.nodes()
	count=0
	for fun in annoDict:
		count+=1
		if (count%500)==0:
			print "Generated "+str(count)+"th functions' SP records..."
		pre=fun[0]
		fungenes=annoDict[fun][2]
		graphfungenes=list(set(graphgenes) & set(fungenes))
		### do not process to retrieve the SP for those functions do not match the specified criteria		
		#if (len(fungenes)>maxFun) or (len(fungenes)<minFun) or (len(graphfungenes)<minFunNet):
		if (len(fungenes)>maxFun) or (len(fungenes)<minFun) or (len(graphfungenes)<2):
			continue

		if not fullgenes:
			subgraph=Graph(graph.subgraph(graphfungenes))

		for i in range(len(graphfungenes)):
			tempG1=graphfungenes[i]
			for j in range((i+1),len(graphfungenes)):
				tempG2=graphfungenes[j]
				#### using all GGfile genes to compute SP
				if fullgenes:
					### only continue if there is a path between tempG1 and tempG2
					if has_path(graph,tempG1,tempG2):
						temppath=shortest_path(graph,tempG1,tempG2)
						funhandlers[pre][fun].write(tempG1+"\t"+tempG2+"\t"+str(len(temppath)-1)+"\t"+','.join(temppath)+"\n")
				else:
				### using only fun genes in GGfile to compute SP
					if has_path(subgraph,tempG1,tempG2):
						temppath=shortest_path(subgraph,tempG1,tempG2)
						funhandlers[pre][fun].write(tempG1+"\t"+tempG2+"\t"+str(len(temppath)-1)+"\t"+','.join(temppath)+"\n")
	
	#################################################################
	### close all the file handlers:
	for fun in annoDict:
		pre=fun[0]
		funhandlers[pre][fun].close()
	
if __name__ == "__main__":
	main()

